@@ -3,26 +3,87 @@ extern crate test;
 
 use std::fs::OpenOptions;
 use std::mem::{size_of, transmute};
+use std::path::PathBuf;
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::{Arc, Mutex, RwLock, RwLockReadGuard};
 
 use anyhow::Result;
-use bytesize::ByteSize;
 use memmap2::MmapMut;
 
 use crate::Cache;
 
-/// Increase the size of the file if needed, and create a memory map from it
-fn resize_and_memmap(
-    filename: &str,
-    index: usize,
+type OnSizeChange = fn(old_size: usize, new_size: usize) -> ();
+
+#[derive(Clone)]
+pub struct DenseFileCacheOpts {
+    filename: Arc<PathBuf>,
+    write: bool,
+    autogrow: bool,
+    init_size: usize,
     page_size: usize,
-    verbose: bool,
-) -> Result<MmapMut> {
-    if page_size % size_of::<usize>() != 0 {
+    on_size_change: Option<OnSizeChange>,
+}
+
+impl DenseFileCacheOpts {
+    pub fn new(filename: PathBuf) -> Self {
+        DenseFileCacheOpts {
+            filename: Arc::new(filename),
+            write: true,
+            autogrow: true,
+            init_size: 1024 * 1024 * 1024, // 1 GB
+            page_size: 1024 * 1024 * 1024, // 1 GB
+            on_size_change: None,
+        }
+    }
+
+    /// Allow data modification
+    pub fn write(mut self, write: bool) -> Self {
+        if !write {
+            todo!("Readonly cache is not supported yet")
+        }
+        self.write = write;
+        self
+    }
+
+    /// Set callback to report when cache size changes
+    pub fn on_size_change(mut self, on_size_change: Option<OnSizeChange>) -> Self {
+        self.on_size_change = on_size_change;
+        self
+    }
+
+    /// Automatically increase cache file size as needed. Ignored for read-only files.
+    pub fn autogrow(mut self, autogrow: bool) -> Self {
+        if !autogrow {
+            todo!("Constant cache size is not supported yet")
+        }
+        self.autogrow = autogrow;
+        self
+    }
+
+    /// Ensure cache file is at least this big. Ignored for read-only files.
+    pub fn init_size(mut self, init_size: usize) -> Self {
+        self.init_size = init_size;
+        self
+    }
+
+    /// When increasing file size, grow it in page size increments. Ignored for read-only files.
+    pub fn page_size(mut self, page_size: usize) -> Self {
+        self.page_size = page_size;
+        self
+    }
+
+    /// Open and initialize cache file.
+    pub fn open(self) -> Result<DenseFileCache> {
+        DenseFileCache::new_opt(self)
+    }
+}
+
+/// Increase the size of the file if needed, and create a memory map from it
+fn resize_and_memmap(index: usize, opts: &DenseFileCacheOpts) -> Result<MmapMut> {
+    if opts.page_size % size_of::<usize>() != 0 {
         panic!(
             "page_size={} is not a multiple of {}.",
-            page_size,
+            opts.page_size,
             size_of::<usize>()
         )
     }
@@ -30,21 +91,15 @@ fn resize_and_memmap(
         .read(true)
         .write(true)
         .create(true)
-        .open(filename)?;
+        .open(opts.filename.as_ref())?;
     let old_size = file.metadata().unwrap().len();
 
     let capacity = (index + 1) * size_of::<usize>();
-    let pages = capacity / page_size + (if capacity % page_size == 0 { 0 } else { 1 });
-    let new_size = (pages * page_size) as u64;
-    // println!("Trying to grow {} ➡ {}", old_size, new_size);
+    let pages = capacity / opts.page_size + (if capacity % opts.page_size == 0 { 0 } else { 1 });
+    let new_size = (pages * opts.page_size) as u64;
     if old_size < new_size {
-        if verbose {
-            println!(
-                "Growing cache {} ➡ {} ({} pages)",
-                ByteSize(old_size),
-                ByteSize(new_size),
-                pages
-            );
+        if let Some(value) = opts.on_size_change {
+            value(old_size as usize, new_size as usize);
         }
         file.set_len(new_size)?;
     }
@@ -63,9 +118,7 @@ fn lock_and_link(memmap: &RwLock<MmapMut>) -> (Option<RwLockReadGuard<MmapMut>>,
 
 #[derive(Clone)]
 pub struct DenseFileCache {
-    filename: Arc<String>,
-    page_size: usize,
-    verbose: bool,
+    opts: DenseFileCacheOpts,
     memmap: Arc<RwLock<MmapMut>>,
     mutex: Arc<Mutex<()>>,
 }
@@ -78,16 +131,14 @@ struct CacheWriter<'a> {
 
 impl DenseFileCache {
     /// Open or create a file for caching
-    pub fn new(filename: String) -> Result<Self> {
-        Self::new_ex(filename, 1024 * 1024 * 1024, true)
+    pub fn new(filename: PathBuf) -> Result<Self> {
+        DenseFileCacheOpts::new(filename).open()
     }
 
-    fn new_ex(filename: String, page_size: usize, verbose: bool) -> Result<Self> {
-        let mmap = resize_and_memmap(&filename, 0, page_size, verbose)?;
+    fn new_opt(opts: DenseFileCacheOpts) -> Result<Self> {
+        let mmap = resize_and_memmap(0, &opts)?;
         Ok(Self {
-            filename: Arc::new(filename),
-            page_size,
-            verbose,
+            opts,
             memmap: Arc::new(RwLock::new(mmap)),
             mutex: Arc::new(Mutex::new(())),
         })
@@ -139,21 +190,16 @@ impl<'a> Cache for CacheWriter<'a> {
             {
                 let _pre_write_lock = self.parent.mutex.lock().unwrap();
                 if index >= self.len() {
-                    // println!("Growing:  Index {}, Length {} ", index, self.len());
                     let p = self.parent;
                     let mut write_lock = p.memmap.write().unwrap();
-                    // println!("Got write lock:  Index {}, Length {} ", index, self.len());
                     write_lock.flush().unwrap();
-                    *write_lock =
-                        resize_and_memmap(&p.filename, index, p.page_size, p.verbose).unwrap();
-                    // println!("Got write lock:  Index {}, Length {} ", index, self.len());
+                    *write_lock = resize_and_memmap(index, &p.opts).unwrap();
                 }
             }
 
             let (mm_setter, raw_data) = lock_and_link(&self.parent.memmap);
             self.mm_setter = mm_setter;
             self.raw_data = raw_data;
-            // println!("Got read lock:  Index {}, Length {} ", index, self.len());
         }
         self.raw_data[index].store(value, Ordering::Relaxed);
     }
@@ -162,6 +208,7 @@ impl<'a> Cache for CacheWriter<'a> {
 #[cfg(test)]
 mod tests {
     use std::fs;
+    use std::path::PathBuf;
 
     use rand::seq::SliceRandom;
     use rand::thread_rng;
@@ -175,7 +222,10 @@ mod tests {
         let test_file = "./dense_file_test.dat";
         let _ = fs::remove_file(test_file);
         {
-            let fc = DenseFileCache::new_ex(test_file.to_string(), 8, false).unwrap();
+            let fc = DenseFileCacheOpts::new(PathBuf::from(test_file))
+                .page_size(8)
+                .open()
+                .unwrap();
             let threads = 10;
             let items = 100000;
             (0_usize..threads)
@@ -210,6 +260,7 @@ mod tests {
 #[cfg(all(feature = "nightly", test))]
 mod bench {
     use std::fs;
+    use std::path::PathBuf;
 
     use crate::*;
 
@@ -219,7 +270,10 @@ mod bench {
     fn dense_bench(bench: &mut Bencher) {
         let test_file = "./dense_file_perf.dat";
         let _ = fs::remove_file(test_file);
-        let fc = DenseFileCache::new_ex(test_file.to_string(), 1024 * 1024, false).unwrap();
+        let fc = DenseFileCacheOpts::new(PathBuf::from(test_file))
+            .page_size(1024 * 1024)
+            .open()
+            .unwrap();
 
         let mut cache = fc.get_accessor();
         bench.iter(|| {
